@@ -12,7 +12,7 @@
 
 lingpg <- function(inc, gender = NULL, id, weight=NULL, sort = NULL,
                    Dom = NULL, period=NULL, dataset = NULL,
-                   na.rm = FALSE, var_name="lin_gpg") {
+                   var_name="lin_gpg") {
 
    ## initializations
 
@@ -58,23 +58,13 @@ lingpg <- function(inc, gender = NULL, id, weight=NULL, sort = NULL,
    if (ncol(inc) != 1) stop("'inc' must be vector or 1 column data.frame, matrix, data.table")
    inc <- inc[,1]
    if(!is.numeric(inc)) stop("'inc' must be a numeric vector")                   
-   if (any(is.na(inc))) warning("'inc' has unknown values")
+   if (any(is.na(inc))) stop("'inc' has unknown values")
 
    # gender
-   if (!is.factor(gender)) stop("'gender' must be a factor.")
-   if (length(levels(gender)) != 2) stop("'gender' must be exactly two levels")
-   if (!all(levels(gender) == c(1, 2))) {
-        gender <- factor(gender, labels=c(1,2))
-        warning("The levels of gender were internally recoded - your first level has to correspond to males")
-       } 
+   if (!is.numeric(gender)) stop("'gender' must be numerical")
    if (length(gender) != n) stop("'gender' must be the same length as 'inc'")
-
-   # id
-   if (is.null(id)) id <- 1:n 
-   id <- data.table(id)
-   if (nrow(id) != n) stop("'id' must be the same length as 'inc'")
-   if (ncol(id) != 1) stop("'id' must be 1 column data.frame, matrix, data.table")
-   if (is.null(names(id))||(names(id)=="id")) names(id) <- "ID"
+   if (length(unique(gender)) != 2) stop("'gender' must be exactly two values")
+   if (!all.equal(unique(gender),c(1, 2))) stop("'gender' must be value 1 for male, 2 for females")
  
    # weight
    weight <- data.frame(weight)
@@ -83,6 +73,7 @@ lingpg <- function(inc, gender = NULL, id, weight=NULL, sort = NULL,
    if (ncol(weight) != 1) stop("'weight' must be vector or 1 column data.frame, matrix, data.table")
    weight <- weight[,1]
    if (!is.numeric(weight)) stop("'weight' must be a numerical")
+   if (any(is.na(weight))) stop("'weight' has unknown values")
  
    # sort
    if (!is.null(sort) && !is.vector(sort) && !is.ordered(sort)) {
@@ -97,7 +88,20 @@ lingpg <- function(inc, gender = NULL, id, weight=NULL, sort = NULL,
                       paste(names(period)[duplicated(names(period))], collapse = ","))
        if (nrow(period) != n) stop("'period' must be the same length as 'inc'")
        if(any(is.na(period))) stop("'period' has unknown values")  
-   }   
+   }
+
+   # id
+   if (is.null(id)) id <- 1:n
+   id <- data.table(id)
+   if (any(is.na(id))) stop("'id' has unknown values")
+   if (ncol(id) != 1) stop("'id' must be 1 column data.frame, matrix, data.table")
+   if (nrow(id) != n) stop("'id' must be the same length as 'inc'")
+   if (is.null(names(id))||(names(id)=="id")) setnames(id,names(id),"ID")
+   if (is.null(period)){ if (any(duplicated(id))) stop("'id' are duplicate values") 
+                       } else {
+                          id1 <- data.table(period, id)
+                          if (any(duplicated(id1))) stop("'id' by period are duplicate values")
+                         }
 
    # Dom     
    if (!is.null(Dom)) {
@@ -110,69 +114,89 @@ lingpg <- function(inc, gender = NULL, id, weight=NULL, sort = NULL,
        }
 
 
-
-    ## computations
-   Dom1 <- Dom
-   if (!is.null(period)) {
-       if (!is.null(Dom1)) { Dom1 <- data.table(period, Dom1)
-        } else Dom1 <- period } 
-
+   ## computations
+   ind0 <- rep.int(1, n)
+   period_agg <- period1 <- NULL
+   if (!is.null(period)) { period1 <- copy(period)
+                           period_agg <- data.table(unique(period))
+                       } else period1 <- data.table(ind=ind0)
+   period1_agg <- data.table(unique(period1))
 
    # GPG by domain (if requested)
+   gpg_id <- id
+   if (!is.null(period)) gpg_id <- data.table(gpg_id, period)
+
+   if(!is.null(Dom)) {
+        Dom_agg <- data.table(unique(Dom))
+        setkeyv(Dom_agg, names(Dom_agg))
+
+        gpg_v <- c()
+        gpg_m <- copy(gpg_id)
 
-   if(!is.null(Dom1)) {
-        Dom_agg <- data.table(unique(Dom1))
-        setkeyv(Dom_agg, names(Dom1))
-        gpg_pr <- c()
-        gpg_id <- id
-        if (!is.null(period)) gpg_id <- data.table(period, gpg_id)
-        gpg_m <- gpg_id
         for(i in 1:nrow(Dom_agg)) {
-               g <- paste(names(Dom), as.matrix(Dom_agg[i,]), sep = ".")
-               breakdown2 <- do.call(paste, as.list(c(g, sep="__")))
-               D <- Dom_agg[i,][rep(1,nrow(Dom1)),]
-               ind <- (rowSums(Dom1 == D) == ncol(Dom1))
-               gpg_l <- linGapCalc(x=inc[ind], gend=gender[ind],
-                                   ids=gpg_id[ind], weights=weight[ind],
-                                   sort=sort[ind], na.rm=na.rm)
-               gpglin <- gpg_l$lin
-               setnames(gpglin, names(gpglin), c(names(gpg_id), paste(var_name, breakdown2, sep="__")))
-               gpg_m <- merge(gpg_m, gpglin, by=names(gpg_id), all.x=T, sort=FALSE)
-               gpg_pr <- rbind(gpg_pr,gpg_l$gpg_pr)                 }
-        colnames(gpg_pr)[ncol(gpg_pr)] <- "gender_pay_gap"
-        gpg_pr <- data.table(Dom_agg, gpg_pr)
-      } else { gpg_l <- linGapCalc(x=inc, gend=gender, ids=id,
-                                   weights=weight, sort=sort,
-                                   na.rm=na.rm)
-               gpg_m <- gpg_l$lin
-               setnames(gpg_m, names(gpg_m), c(names(id), var_name))
-               gpg_pr <- gpg_l$gpg_pr 
-               setnames(gpg_pr, names(gpg_pr)[ncol(gpg_pr)], "gender_pay_gap")
-       }
+            g <- c(var_name, paste(names(Dom), as.matrix(Dom_agg[i,]), sep = "."))
+            var_nams <- do.call(paste, as.list(c(g, sep="__")))
+            indi <- (rowSums(Dom == Dom_agg[i,][ind0,]) == ncol(Dom))
+            
+            gpg_l <- lapply(1:nrow(period1_agg), function(j) {
+                 indj <- ((rowSums(period1 == period1_agg[j,][ind0,]) == ncol(period1))&(indi))
+                 if (!is.null(period)) { rown <- cbind(period_agg[j], Dom_agg[i])
+                                     } else rown <- Dom_agg[i] 
+                 if (!all(indj)) {
+                      gpgl <- linGapCalc(x=inc[indj], gend=gender[indj],
+                                         ids=gpg_id[indj], weights=weight[indj],
+                                         sort=sort[indj])
+                      list(data.table(rown, gpg=gpgl$gpg_pr), gpgl$lin)
+                     }  else list(data.table(rown, gpg=0, Gini_eu=0),                                            
+                                lin=data.table(lin=gpg_id[indi], lin=0))
+              })
+
+            gpgs <- rbindlist(lapply(gpg_l, function(x) x[[1]]))
+            gpglin <- rbindlist(lapply(gpg_l, function(x) x[[2]]))
+
+            setnames(gpglin, names(gpglin), c(names(gpg_id), var_nams))
+            setkeyv(gpg_m, names(gpg_id))
+            setkeyv(gpglin, names(gpg_id))
+            gpg_m <- merge(gpg_m, gpglin, all.x=T)
+            gpg_v <- rbind(gpg_v, gpgs) 
+          }
+      } else { gpg_l <- lapply(1:nrow(period1_agg), function(j) {
+                           if (!is.null(period)) { 
+                                         rown <- period_agg[j]
+                                         setkeyv(rown, names(rown))
+                                         rown <- merge(rown, quantile, all.x=TRUE)
+                                       } else rown <- quantile
+                           indj <- (rowSums(period1 == period1_agg[j,][ind0,]) == ncol(period1))
+      
+                           gpg_l <- linGapCalc(x=inc[indj], gend=gender[indj],
+                                               ids=gpg_id[indj], weights=weight[indj],
+                                               sort=sort[indj])
+
+                          if (!is.null(period)) { 
+                                   gpgs <- data.table(period_agg[j], gpg=gpg_l$gpg)
+                             } else gpgs <- data.table(gpg=gpg_l$gpg)
+                          list(gpg=gpgs, lin=gpg_l$lin)
+                       })
+               gpg_v <- rbindlist(lapply(gpg_l, function(x) x[[1]]))
+               gpg_m <- rbindlist(lapply(gpg_l, function(x) x[[2]]))
+               setnames(gpg_m, names(gpg_m), c(names(gpg_id), var_name))
+            } 
     gpg_m[is.na(gpg_m)] <- 0  
-    return(list(value=gpg_pr, lin=gpg_m))
+    setkeyv(gpg_m, names(gpg_id))
+    return(list(value=gpg_v, lin=gpg_m))
 }
 
+
   ## workhorse
-linGapCalc <- function(x, gend, ids, weights = NULL, sort = NULL, na.rm = FALSE) {
+linGapCalc <- function(x, gend, ids, weights = NULL, sort = NULL) {
     if(is.null(gend)) stop("'gender' must be supplied")
     if (length(gend)!=length(x)) stop("'x' is not the same as 'gend'")
     if (length(gend)!=length(weights)) stop("'weights' is not the same as 'gend'")
-    # initializations
-    if (isTRUE(na.rm)){
-          indices <- !is.na(x)
-          x <- x[indices]
-          ids <- ids[indices]
-          gend <- gend[indices]
-          if (!is.null(weights)) weights <- weights[indices]
-          if (!is.null(id)) id <- id[indices]
-          if (!is.null(sort)) sort <- sort[indices]
-      } else if(any(is.na(x))) return(NA)
-    
+   
     if (is.null(weights)) weights <- rep.int(1, length(x))  # equal weights
 
-    indic_men <- ifelse((gend==1)&(!is.na(x)),1,0)
-    indic_women <- ifelse((gend==2)&(!is.na(x)),1,0)
+    indic_men <- ifelse(gend==1, 1, 0)
+    indic_women <- ifelse(gend==2, 1, 0)
  
     x[is.na(x)] <- 0
    
@@ -190,8 +214,9 @@ linGapCalc <- function(x, gend, ids, weights = NULL, sort = NULL, na.rm = FALSE)
    lin <- 100*(1-gpg)*((indic_men/Nwomen)-(indic_men/Nmen)+((x*indic_men)/SINCmen)-((x*indic_women)/SINCwomen))
  #-----------------------------------------------------------------------------
 
-   lin_id <- cbind(ids,lin)
+   lin_id <- cbind(ids, lin)
 
-   return(list(gpg_pr=gpg_pr,lin=lin_id))
+   gpg <- data.table(gpg=gpg)
+   return(list(gpg_pr=gpg_pr, lin=lin_id))
 }
 
