@@ -2,12 +2,12 @@
 transpos <- function(variable, period_NULL, valnames, pernames, variabname=NULL) {
         if (period_NULL) {dati <- data.table(nv=names(variable), t(variable))
                           setnames(dati, names(dati), c("variable", valnames))
-                  } else { dati <- data.table(melt(variable, id=c(pernames)))
-                           setnames(dati, names(dati)[ncol(dati)], valnames)
-                     } 
+                 } else { dati <- melt(variable, id=c(pernames))
+                          setnames(dati, names(dati)[ncol(dati)], valnames)
+                   } 
        dati[, variable:=as.character(variable)]
        if (!is.null(variabname)) { setnames(dati, "variable", variabname)
-                                } else variabname <- "variable"
+                            } else variabname <- "variable"
        setkeyv(dati, c(pernames, variabname)) 
        return(dati)
 }
@@ -134,7 +134,6 @@ vardom <- function(Y, H, PSU, w_final,
   if (!is.numeric(w_final)) stop("'w_final' must be numerical")
   if (any(is.na(w_final))) stop("'w_final' has unknown values") 
 
-
   # period     
   if (!is.null(period)) {
       period <- data.table(period)
@@ -154,10 +153,7 @@ vardom <- function(Y, H, PSU, w_final,
   if (nrow(id) != n) stop("'id' length must be equal with 'Y' row count")
   if (is.null(names(id))||(names(id)=="id")) setnames(id,names(id),"ID")
   if (is.null(period)){ if (any(duplicated(id))) stop("'id' are duplicate values") 
-                      } else {
-                         id1 <- data.table(period, id)
-                         if (any(duplicated(id1))) stop("'id' by period are duplicate values")
-                        }
+                       } else if (any(duplicated(data.table(period, id)))) stop("'id' by period are duplicate values")
 
   # N_h
   if (!is.null(N_h)) {
@@ -167,6 +163,9 @@ vardom <- function(Y, H, PSU, w_final,
       if (any(is.na(N_h))) stop("'N_h' has unknown values") 
       if (is.null(names(N_h))) stop("'N_h' must be colnames")
       if (is.null(names(N_h))) stop("'N_h' must be colnames")
+      namesH <- names(H)
+      if (H[, class(get(namesH))]!=N_h[, class(get(namesH))]) 
+                                         stop("Strata class for 'H' and 'N_h' is not equal ")
 
       if (is.null(period)) {
              if (names(H) != names(N_h)[1]) stop("Strata titles for 'H' and 'N_h' is not equal")
@@ -174,6 +173,9 @@ vardom <- function(Y, H, PSU, w_final,
              if (any(duplicated(N_h[, head(names(N_h),-1), with=F]))) stop("Strata values for 'N_h' must be unique")
        } else { pH <- data.frame(period, H)
                 if (any(names(pH) != names(N_h)[c(1:(1+np))])) stop("Strata titles for 'period' with 'H' and 'N_h' is not equal")
+                nperH <- names(period)
+                if (H[, class(get(nperH))]!=N_h[, class(get(nperH))]) 
+                                                       stop("Period class for 'period' and 'N_h' is not equal ")
                 if (any(is.na(merge(unique(pH), N_h, by=names(pH), all.x = T)))) stop("'N_h' is not defined for all stratas and periods")
                 if (any(duplicated(N_h[, head(names(N_h),-1), with=F]))) stop("Strata values for 'N_h' must be unique in all periods")
                 pH <- NULL
@@ -261,22 +263,25 @@ vardom <- function(Y, H, PSU, w_final,
       
   # Domains
   if (!is.null(Dom)) Y1 <- domain(Y, Dom) else Y1 <- Y
-  n_nonzero <- Y1[, lapply(.SD, function(x)  as.numeric(x!=0)), .SDcols = names(Y1)]
+  n_nonzero <- copy(Y1)
   if (!is.null(period)){ n_nonzero <- data.table(period, n_nonzero) 
-                         n_nonzero <- n_nonzero[, lapply(.SD, sum), keyby=names(period), .SDcols=names(Y1)]
-                  } else n_nonzero <- n_nonzero[, lapply(.SD, sum), .SDcols=names(Y1)]
+                         n_nonzero <- n_nonzero[, lapply(.SD, function(x) 
+                                                          sum(as.integer(abs(x)> .Machine$double.eps))),
+                                                          keyby=names(period),
+                                                         .SDcols = names(Y1)]
+                  } else n_nonzero <- n_nonzero[, lapply(.SD, function(x) 
+                                                          sum(as.integer(abs(x)> .Machine$double.eps))),
+                                                         .SDcols = names(Y1)]
 
   sample_size <- pop_size <- NULL
-  if (!is.null(Dom)) { if (!is.null(period)) {nhs <- data.table(Dom, period, sample_size=1, pop_size=w_final)
-                                              nhs <-  nhs[, lapply(.SD, sum, na.rm=T),
-                                                                    keyby=c(names(Dom), names(period)),
-                                                                   .SDcols=c("sample_size", "pop_size")]
-                                     } else { nhs <- data.table(Dom, sample_size=1, pop_size=w_final)
-                                              nhs <-  nhs[, lapply(.SD, sum, na.rm=T),
-                                                                    keyby=names(Dom),
-                                                                   .SDcols=c("sample_size", "pop_size")]
-                                  }
-                           } else nhs <- data.table(sample_size=nrow(Y1), pop_size=sum(w_final)) 
+  nhs <- data.table(sample_size=1, pop_size=w_final)
+  if (!is.null(period)) nhs <- data.table(period, nhs)
+  if (!is.null(Dom)) nhs <- data.table(Dom, nhs)
+  if (!is.null(c(Dom, period))) {nhs <- nhs[, lapply(.SD, sum, na.rm=T),
+                                                      keyby=eval(names(nhs)[0:1-ncol(nhs)]),
+                                                     .SDcols=c("sample_size", "pop_size")]
+                          } else nhs <- nhs[, lapply(.SD, sum, na.rm=T),
+                                                     .SDcols=c("sample_size", "pop_size")]
 
   # Design weights
   if (!is.null(X)) w_design <- w_final / g else w_design <- w_final
@@ -407,7 +412,7 @@ vardom <- function(Y, H, PSU, w_final,
                                 Z_nov <- hZ[, lapply(.SD, sum, na.rm = T), keyby=names(period), .SDcols = names(Z1)]
                               }
          Z_nov <- transpos(Z_nov, is.null(period), "Z_nov", names(period), "variableDZ")
-         setkeyv(all_result, "variableDZ")
+         setkeyv(all_result, c(names(period), "variableDZ"))
          all_result <- merge(all_result, Z_nov, all=T)                                            
       }
 
@@ -420,7 +425,7 @@ vardom <- function(Y, H, PSU, w_final,
   Z_nov <- hY <- hZ <- YZnames <- dati <- NULL                          
 
   all_result[, estim:=Y_nov]
-  if (!is.null(Z_nov)) { all_result[, estim:=Y_nov/Z_nov] }
+  if (!is.null(all_result$Z_nov)) all_result[, estim:=Y_nov/Z_nov] 
 
   if (nrow(all_result[var_est < 0])>0) stop("Estimation of variance are negative!")
  
@@ -476,12 +481,12 @@ vardom <- function(Y, H, PSU, w_final,
   all_result <- merge(nosr, all_result)
   nosr <- nosr1 <- NULL
   
-  if (!is.null(Z_nov)) {all_result[, variable:=paste("R", get("variable"), sep="__", get("variableZ"))] } 
+  if (!is.null(all_result$Z_nov)) {all_result[, variable:=paste("R", get("variable"), sep="__", get("variableZ"))] } 
   setkeyv(all_result, c(names(Dom), names(period)))
 
-  if (!is.null(Dom)) { all_result <- merge(all_result, nhs, all=T)
-                           } else { all_result[, sample_size:=nhs$sample_size]
-                                       all_result[, pop_size:=nhs$pop_size] } 
+  if (!is.null(c(Dom, period))) { all_result <- merge(all_result, nhs, all=T)
+                         } else { all_result[, sample_size:=nhs$sample_size]
+                                  all_result[, pop_size:=nhs$pop_size] } 
 
   variab <- c("sample_size", "n_nonzero", "pop_size", "estim", "var", "se", 
               "rse", "cv", "absolute_margin_of_error", "relative_margin_of_error",
